#ifndef AD5662_h
#define AD5662_h
#include <utility/DigitalPin.h>
//------------------------------------------------------------------------------
template<uint8_t SyncPin, uint8_t DinPin, uint8_t SclkPin, bool WaveFmt = false>
class AD5662 {
 public:
  void begin() {
    fastPinMode(SclkPin, 1);
    fastDigitalWrite(SclkPin, 1);
    fastPinMode(SyncPin, 1);
    fastDigitalWrite(SyncPin, 1);
    fastPinMode(DinPin, 1);
  } 
  //----------------------------------------------------------------------------
  inline __attribute__((always_inline))   
  void write(uint16_t v) {
    // Set data low.
    fastDigitalWrite(DinPin, 0);
    fastDigitalWrite(SyncPin, 0);
    // send eight zeros
    dacClockBit();
    dacClockBit();
    dacClockBit();
    dacClockBit();
    dacClockBit();
    dacClockBit();
    dacClockBit();
    dacClockBit();
    dacWriteBit(v, 15, WaveFmt);
    dacWriteBit(v, 14);
    dacWriteBit(v, 13);
    dacWriteBit(v, 12);
    dacWriteBit(v, 11);
    dacWriteBit(v, 10);
    dacWriteBit(v,  9);
    dacWriteBit(v,  8);
    dacWriteBit(v,  7);
    dacWriteBit(v,  6);
    dacWriteBit(v,  5);
    dacWriteBit(v,  4);
    dacWriteBit(v,  3);
    dacWriteBit(v,  2);
    dacWriteBit(v,  1);
    dacWriteBit(v,  0);
    fastDigitalWrite(SyncPin, 1);
  }  
 private:  
  //------------------------------------------------------------------------------
  // Low going pulse to DAC.
  inline __attribute__((always_inline)) 
  void dacClockBit() {
    fastDigitalWrite(SclkPin, 0);
    fastDigitalWrite(SclkPin, 1);
  }
  //------------------------------------------------------------------------------
  // Send one bit.  Use complement if c is true.
  inline __attribute__((always_inline)) 
  void dacWriteBit(uint16_t v, uint16_t b, bool c = false) {
    fastDigitalWrite(DinPin, c != (bool)(v & (1 << b)));
    dacClockBit();
  }  
};
 #endif  // AD5662_h