#ifndef AD5662_USART1_h
#define AD5662_USART1_h
#include <avr/io.h>
#if defined(__AVR_ATmega1284P__)\
|| defined(__AVR_ATmega1284__)\
|| defined(__AVR_ATmega644P__)
#define USARTDDR DDRD // The data direction register for the USART XCK1 line.
#define XCKPIN PD4    //  The bit position for the USART XCK1 line.
#else  // 1284 or 644
#error  XCK1 not defined
#endif // 1284 or 644
//------------------------------------------------------------------------------
template<uint8_t CsPin, bool WaveFmt = false>
class AD5662_USART1 {
 public:
  void begin(uint8_t spiMode = 2, uint16_t brReg = 0) {
    fastPinMode(CsPin, 1);
    fastDigitalWrite(CsPin, 1);
    init_usart1_spi(spiMode, brReg);
  } 
  //----------------------------------------------------------------------------
  inline __attribute__((always_inline))   
  void write(uint16_t v) {
    fastDigitalWrite(CsPin, 0);
    if (WaveFmt) {
      v ^= 0X8000;
    }
    UDR1 = 0;
    UDR1 = v >> 8;
    // clear transmit complete flag
    UCSR1A = 1 << TXC1;
    // wait for Transmit buffer space
    while (!(UCSR1A & (1 << UDRE1)));
    UDR1 = v;
    //Wait for transmit complete
    while (!(UCSR1A & (1 << TXC1)));
    
    fastDigitalWrite(CsPin, 1);
  }
private:
  //----------------------------------------------------------------------------
  /*! \brief  Initialize USART as SPI Master.
   *
   *  Initialize on-chip USART1 in Master SPI Mode and set the
   *  required port directions. There are no other masters on the
   *  bus, so the XCK line is configured as output.
   *
   *  \param spimode  SPI mode. Must be 0, 1, 2 or 3.
   *  \param brreg  SPI speed. Value to put into USART Baud Rate Register.
   *
   *  \note  SPI speed in bits per second: bps = CPUFREQ/(2*(brreg+1)).
   *  \note  brreg value: brreg = (CPUFREQ/(2*bps))-1.
   */
  static void init_usart1_spi(uint8_t spimode, uint16_t brreg ){
    // Baud rate must be set to 0 prior to enabling the USART as SPI
    // master, to ensure proper initialization of the XCK line.
    UBRR1 = 0;
    
    // Set XCK line to output, ie. set USART in master mode.
    USARTDDR |= (1 << XCKPIN);
    
    // Set USART to Master SPI mode.
    UCSR1C = (1 << UMSEL11) | (1 << UMSEL10);
    
    // Set clock polarity and phase to correct SPI mode.
    if (spimode & 0x01) UCSR1C |= (1 << UCPOL1);
    if (spimode & 0x02) UCSR1C |= (1 << UCPHA1);
    
    // Enable RX and TX.
    UCSR1B = (1 << RXEN1) | (1 << TXEN1);
    
    // Set baud rate. Must be set _after_ enabling the transmitter.
    UBRR1 = brreg;
  } 
};

#endif AD5662_USART1_h