#ifndef AD7680_h
#define AD7680_h
#include <utility/DigitalPin.h>
//------------------------------------------------------------------------------
/** nop to tune soft SPI timing */
#ifndef nop
#define nop asm volatile ("nop\n\t")
#endif  // nop
//------------------------------------------------------------------------------
template<uint8_t CsPin, uint8_t SdataPin, uint8_t SclkPin, bool WaveFmt = false>
class AD7680 {
 public:
  //----------------------------------------------------------------------------
  /** Set pin modes and initial levels. */  
  void begin() {
    fastPinMode(SclkPin, 1);
    fastDigitalWrite(SclkPin, 0);
    fastPinMode(CsPin, 1);
    fastDigitalWrite(CsPin, 1);
    fastPinMode(SdataPin, 0);
  }
  //----------------------------------------------------------------------------
  /** Read AD7680 16-bit ADC in less than 8 microseconds
   * \return 16-bit value read.
   */
  inline __attribute__((always_inline)) 
  uint16_t read() {
    fastDigitalWrite(CsPin, 0);
    uint16_t v = 0;
    fastDummy(1);
    fastDummy(0);
    fastDummy(0);
    fastDummy(0);
    readBit(v,  15);
    readBit(v,  14);
    readBit(v,  13);
    readBit(v,  12);
    readBit(v,  11);
    readBit(v,  10);
    readBit(v,  9);
    readBit(v,  8);
    readBit(v,  7);
    readBit(v,  6);
    readBit(v,  5);
    readBit(v,  4);
    readBit(v,  3);
    readBit(v,  2);
    readBit(v,  1);
    readBit(v,  0);
    fastDigitalWrite(CsPin, 1);
    return WaveFmt ? v ^ 0X8000 : v;
  }
 private:
  //----------------------------------------------------------------------------

  /** clock for don't care bits
   *\param[in] delayBefore if true, delay before raising clock*/
  inline __attribute__((always_inline)) 
  void fastDummy(bool first) {
    if (!first) nop;
    fastDigitalWrite(SclkPin, 1);
    nop;
    fastDigitalWrite(SclkPin, 0);
  }
  //----------------------------------------------------------------------------
  inline __attribute__((always_inline))
  void readBit(uint16_t &v, uint8_t b) {
    fastDigitalWrite(SclkPin, 1);
    if (fastDigitalRead(SdataPin)) v |= (1 << b);
    fastDigitalWrite(SclkPin, 0);
   }
}; 
#endif  // AD7680_h