#ifndef CrossRoads_h
#define CrossRoads_h
#include <AD5662.h>
#include <AD5662_USART1.h>
#include <AD7680.h>
#include <MCP3201.h>
#include <MCP4921.h>
#include <MCP4921_USART1.h>
#include <WaveFile.h>
//------------------------------------------------------------------------------
// Pin definitions.
// pins for 1284 Bobuino - uses correct pin map in utility/DigitalPin.h
//#error Define pins for BOBUINO and then remove this line.
//
// ADC is bit-bang so any pins can be used
const uint8_t ADC_SCLK_PIN  =  5;  // PB1
const uint8_t ADC_SDATA_PIN =  6;  // PB2
const uint8_t ADC_CS_PIN    =  7;  // PB3
//
// DAC is on USART 1 so DIN and SCLK are fixed.  SYNC can be any pin.
const uint8_t DAC_DIN_PIN   =  3;  // PD3
const uint8_t DAC_SCLK_PIN  = 30;  // PD4
const uint8_t DAC_SYNC_PIN  =  8;  // PD5
//------------------------------------------------------------------------------
/** doxygen can't handle __attribute__ very well */
#define STATIC_ALWAYS_INLINE static inline __attribute__((always_inline))
//------------------------------------------------------------------------------
/** setup 16 bit timer1 */
STATIC_ALWAYS_INLINE void timer1Rate(uint16_t rate) {
  // no pwm
  TCCR1A = 0;
  // no clock div, CTC mode
  TCCR1B = _BV(WGM13) | _BV(WGM12) | _BV(CS10);
  // set TOP for timer reset
  ICR1 = F_CPU/(uint32_t)rate;
  // compare value for ADC interrupt
  OCR1A =  16;
  // compare value for DAC interrupt
  OCR1B = 16;
}
//------------------------------------------------------------------------------
/** Start ADC interrupt */
STATIC_ALWAYS_INLINE void adcStart() {
  TIMSK1 |= (1 << OCIE1A);  // enable ADC timer1 cmp A interrupt
}
//------------------------------------------------------------------------------
/** Stop ADC interrupt */
STATIC_ALWAYS_INLINE void adcStop() {
  TIMSK1 &= ~(1 << OCIE1A);  // disable ADC timer interrupt
}
//------------------------------------------------------------------------------
/** Start DAC interrupt */
STATIC_ALWAYS_INLINE void dacStart() {
  TIMSK1 |= (1 << OCIE1B);  // enable DAC timer1 cmp B interrupt
}
//------------------------------------------------------------------------------
/** Stop DAC interrupt */
STATIC_ALWAYS_INLINE void dacStop() {
  TIMSK1 &= ~(1 << OCIE1B);  // disable DAC timer interrupt
}
#endif  // CrossRoads_h