#ifndef MCP3201_h
#define MCP3201_h
#include <utility/DigitalPin.h>
//------------------------------------------------------------------------------
/** nop to tune soft SPI timing */
#ifndef nop
#define nop asm volatile ("nop\n\t")
#endif  // nop
//------------------------------------------------------------------------------
template<uint8_t CsPin, uint8_t DoutPin, uint8_t ClkPin, bool WaveFmt = false>
class MCP3201 {
 public:
  //----------------------------------------------------------------------------
  /** Set pin modes and initial levels. */
  void begin() {
    fastPinMode(ClkPin, 1);
    fastDigitalWrite(ClkPin, 0);
    fastPinMode(CsPin, 1);
    fastDigitalWrite(CsPin, 1);
    fastPinMode(DoutPin, 0);
  }
  //----------------------------------------------------------------------------
  /** Read MCP3201 12-bit ADC in less than 8 microseconds
   * \return 16-bit value read.
   */
  inline __attribute__((always_inline)) 
  uint16_t read() {
    fastDigitalWrite(CsPin, 0);
    uint16_t v = 0;
    // Sample
    mcpAdcDummy(false);
    // Sample
    mcpAdcDummy(true);
    // NULL BIT
    mcpAdcDummy(true);
    if (WaveFmt) {
      readBit(v,  15);
      readBit(v,  14);
      readBit(v,  13);
      readBit(v,  12);
    }
    readBit(v,  11);
    readBit(v,  10);
    readBit(v,  9);
    readBit(v,  8);
    readBit(v,  7);
    readBit(v,  6);
    readBit(v,  5);
    readBit(v,  4);
    if (!WaveFmt) {
      readBit(v,  3);
      readBit(v,  2);
      readBit(v,  1);
      readBit(v,  0);
    }
    fastDigitalWrite(CsPin, 1);
    return WaveFmt ? v ^ 0X8000 : v;
  }
 private:
  //------------------------------------------------------------------------------
  /** delay n nops
   * \param[in] n nops to delay, must be a constant so compiler optimizes if().
   */
  inline __attribute__((always_inline)) 
  void mcpDelay(uint8_t n) {
    nop;
    if (n > 1) nop;
    if (n > 2) nop;
    if (n > 3) nop;
  }
  //----------------------------------------------------------------------------
  /** clock for don't care bits
   *\param[in] delayBefore if true, delay before raising clock*/
  inline __attribute__((always_inline)) 
  void mcpAdcDummy(bool delayBefore) {
    if (delayBefore) mcpDelay(3);
    fastDigitalWrite(ClkPin, 1);
    mcpDelay(3);
    fastDigitalWrite(ClkPin, 0);
  }
  //----------------------------------------------------------------------------
  inline __attribute__((always_inline))
  void readBit(uint16_t &v, uint8_t b) {
    mcpDelay(3);
    fastDigitalWrite(ClkPin, 1);
    nop;
    if (fastDigitalRead(DoutPin)) v |= (1 << b);
    fastDigitalWrite(ClkPin, 0);
   }
}; 
#endif  // MCP3201_h