#ifndef MCP4921_h
#define MCP4921_h
#include <utility/DigitalPin.h>
//------------------------------------------------------------------------------
template<uint8_t CsPin, uint8_t SdiPin, uint8_t SckPin, bool WaveFmt = false>
class MCP4921 {
 public:
  void begin() {
    fastPinMode(SckPin, 1);
    fastDigitalWrite(SckPin, 1);
    fastPinMode(CsPin, 1);
    fastDigitalWrite(CsPin, 1);
    fastPinMode(SdiPin, 1);
  } 
  //----------------------------------------------------------------------------
  inline __attribute__((always_inline))   
  void write(uint16_t v) {
    fastDigitalWrite(CsPin, 0);
    fastDigitalWrite(SdiPin, 0);
    dacClockBit();  // DAC A
    dacClockBit();  // unbuffered
    fastDigitalWrite(SdiPin, 1);
    dacClockBit();  // 1X gain
    dacClockBit();  // no SHDN
    if (WaveFmt) {
      dacWriteBit(v, 15, true);
      dacWriteBit(v, 14);
      dacWriteBit(v, 13);
      dacWriteBit(v, 12);
    }
    dacWriteBit(v, 11);
    dacWriteBit(v, 10);
    dacWriteBit(v,  9);
    dacWriteBit(v,  8);
    dacWriteBit(v,  7);
    dacWriteBit(v,  6);
    dacWriteBit(v,  5);
    dacWriteBit(v,  4);
    if (!WaveFmt) {
      dacWriteBit(v,  3);
      dacWriteBit(v,  2);
      dacWriteBit(v,  1);
      dacWriteBit(v,  0);
    }
    fastDigitalWrite(CsPin, 1);
  }  
 private:  
  //----------------------------------------------------------------------------
  // Low going pulse to DAC.
  inline __attribute__((always_inline)) 
  void dacClockBit() {
    fastDigitalWrite(SckPin, 0);
    fastDigitalWrite(SckPin, 1);
  }
  //----------------------------------------------------------------------------
  // Send one bit.  Use complement if c is true.
  inline __attribute__((always_inline)) 
  void dacWriteBit(uint16_t v, uint16_t b, bool c = false) {
    fastDigitalWrite(SdiPin, c != (bool)(v & (1 << b)));
    dacClockBit();
  }  
};
 #endif  // MCP4921_h