// Output all DAC levels and record value read by ADC.
// Connect the DAC output to the ADC input.

#include <CrossRoads.h>
#include <SdFat.h>

// Set the next line for the correct SD chip select pin.
const uint8_t SD_CS_PIN = SS;

// Write signed Wave format files if USE_WAVE_FORMAT is nonzero.
#define USE_WAVE_FORMAT 1
//------------------------------------------------------------------------------
//#define DBG_12_BIT_TEST
#ifndef DBG_12_BIT_TEST
AD7680<ADC_CS_PIN, ADC_SDATA_PIN, ADC_SCLK_PIN, USE_WAVE_FORMAT> adc;
AD5662_USART1<DAC_SYNC_PIN, USE_WAVE_FORMAT> dac;
// bit-bang DAC driver
//AD5662<DAC_SYNC_PIN, DAC_DIN_PIN, DAC_SCLK_PIN, USE_WAVE_FORMAT> dac;
#else  // DBG_12_BIT_TEST
MCP3201<ADC_CS_PIN, ADC_SDATA_PIN, ADC_SCLK_PIN, USE_WAVE_FORMAT> adc;
MCP4921_USART1<DAC_SYNC_PIN, USE_WAVE_FORMAT> dac;
// bit-bang DAC driver
//MCP4921<DAC_SYNC_PIN, DAC_DIN_PIN, DAC_SCLK_PIN, USE_WAVE_FORMAT> dac;
#endif  // DBG_12_BIT_TEST

SdFat sd;
SdFile file;

// Name must include two zeros. 
char name[] = "LOOP00.CSV";

#if USE_WAVE_FORMAT
typedef int16_t data_t;
const data_t DAC_BEGIN = 0X8000;
const data_t DAC_END = 0X7FFF;
#else  // USE_WAVE_FORMAT
typedef uint16_t data_t;
const data_t DAC_BEGIN = 0;
const data_t DAC_END = 0XFFFF;
#endif  // USE_WAVE_FORMAT
//------------------------------------------------------------------------------
void setup() {
  Serial.begin(9600);
  Serial.println("Type any character");
  while (!Serial.available());
  if (!sd.begin()) {
    Serial.println("sd.begin()");
    return;
  }
  uint8_t nn;
  // Find rotation number.
  for (nn = 0; nn < 7 && name[nn] != '0'; nn++);
  
  for (uint8_t n = 0; n < 100; n++) {
    name[nn] = '0' + n/10;
    name[nn + 1] = '0' + n%10;
    if (file.open(name, O_WRITE | O_CREAT | O_EXCL)) break;
  }
  if (!file.isOpen()) {
    Serial.println("file.open");
    return;
  }
  Serial.print("Writing ");
  Serial.println(name);
  adc.begin();
  dac.begin();
  file.println("DAC,ADC");
  data_t dacValue = DAC_BEGIN;
  while(1) {
    dac.write(dacValue);
    // Allow DAC to settle.
    delayMicroseconds(10);
    data_t adcValue = adc.read();
    file.printField(dacValue, ',');
    file.printField(adcValue, '\n');
    if (dacValue++ == DAC_END) break;
  };
  
  file.close();
  Serial.println("Done");
}
//------------------------------------------------------------------------------
void loop() {}